﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace mtpcs
{
    public partial class FormMain : Form
    {
        public FormMain()
        {
            InitializeComponent();
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            SetDevice();
        }

        private string CurrentPath = "";
        private nMtp.FileInfo [] FileInfoList;

        /// <summary>
        /// デバイス一覧取得
        /// </summary>
        private void SetDevice()
        {
            string[] device_list = nMtp.Directory.GetDevices();
            foreach (string device in device_list)
            {
                TreeNode node = treeFolder.Nodes.Add(device);
                node.ImageIndex = 0;
                node.SelectedImageIndex = 0;
                node.Tag = device;
                SetFolder(node, device);
            }
        }

        /// <summary>
        /// フォルダツリー作成
        /// </summary>
        /// <param name="node">TreeViewノード</param>
        /// <param name="path">デバイスパス名</param>
        private void SetFolder(TreeNode node, string path)
        {
            try {
                nMtp.DirectoryInfo dir = new nMtp.DirectoryInfo(path);
                nMtp.DirectoryInfo [] folder_list = dir.GetDirectories();
                foreach (nMtp.DirectoryInfo folder in folder_list)
                {
                    TreeNode sub_node = node.Nodes.Add(folder.Name);
                    sub_node.ImageIndex = 1;
                    sub_node.SelectedImageIndex = 1;
                    sub_node.Tag = folder.FullName;
                    SetFolder(sub_node, folder.FullName);
                }
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
            }
        }
        /// <summary>
        /// フォルダツリー選択後処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void treeFolder_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (treeFolder.SelectedNode.Tag != null)
            {
                // カレントパスを取得
                CurrentPath = (string)treeFolder.SelectedNode.Tag;
                // ファイル一覧更新
                GetFile(CurrentPath);
            }

        }
        /// <summary>
        /// ファイル一覧表示
        /// </summary>
        /// <param name="path">カレントパス</param>
        private void GetFile(string path)
        {
            listFile.VirtualListSize = 0;
            try {
                nMtp.DirectoryInfo dir = new nMtp.DirectoryInfo(path);
                FileInfoList = dir.GetFiles();
                listFile.VirtualListSize = FileInfoList.Length;
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
            }
        }

        private const int itemSrc = 0;
        private const int itemDst = 1;
        private const int itemResult = 2;
        private const int itemError = 3;
        private const int itemMax = 4;
        /// <summary>
        /// フォルダコピー
        /// </summary>
        /// <param name="flag">コピー設定</param>
        private void FolderCopy(int flag)
        {
            // コピー先フォルダ選択
            nMtp.FolderBrowserDialog dialog = new nMtp.FolderBrowserDialog();
            dialog.Description = "コピーするフォルダを選択してください";
            if(dialog.ShowDialog() == DialogResult.OK) {
                nMtp.DirectoryInfo dir = new nMtp.DirectoryInfo(CurrentPath);
                try {
                    // コピーファイル情報取得
                    dir.CopyFirst(dialog.SelectPath, flag);
                    // 結果ダイアログ生成
                    FormResult form_result = new FormResult();
                    string[] item = new string[itemMax];
                    int count = 0;
                    // 進捗表示ダイアログ生成
                    FormProgress form_progress = new FormProgress();
                    form_progress.Owner = this;
                    form_progress.SetPos(count);
                    form_progress.SetMax(dir.CopyCount);
                    form_progress.Show();
                    Application.DoEvents();
                    // 一つずつファイルコピー
                    while(dir.CopyNext()) {
                        // 結果を追加
                        item[itemSrc] = dir.CopySourceFile;
                        item[itemDst] = dir.CopyDestFile;
                        form_progress.SetMessage(item[itemSrc], item[itemDst]);
                        form_progress.SetPos(++count);
                        Application.DoEvents();
                        if(dir.CopyResult) {
                            item[itemResult] = "成功";
                            item[itemError] = "";
                        } else {
                            item[itemResult] = "失敗";
                            item[itemError] = dir.CopyErrorString;
                        }
                        form_result.AddItem(item);
                        // 中止ボタンが押されていれば中止
                        if(form_progress.Stop) {
                            break;
                        }
                    }
                    form_progress.Close();
                    dir.CopyClose();
                    // 結果ダイアログ表示
                    form_result.ShowDialog(this);
                } catch (Exception ex) {
                    MessageBox.Show("エラー:" + ex.Message);
                }
            }
        }

        private void menuItemFolderCopy_Click(object sender, EventArgs e)
        {
            // フォルダコピー カレントフォルダのみ
            FolderCopy(0);
        }

        private void MenuItemSubFolderCopy_Click(object sender, EventArgs e)
        {
            // フォルダコピー サブフォルダもコピー
            FolderCopy(nMtp.Directory.CopySubFolder);
        }

        /// <summary>
        /// ファイルコピー
        /// </summary>
        /// <param name="flag">コピー設定</param>
        private void FileCopy(int flag)
        {
            if(listFile.SelectedIndices.Count > 0) {
                // コピー先フォルダ選択
                nMtp.FolderBrowserDialog dialog = new nMtp.FolderBrowserDialog();
                dialog.Description = "コピーするフォルダを選択してください";
                if(dialog.ShowDialog() == DialogResult.OK) {
                    // 結果ダイアログ生成
                    FormResult form_result = new FormResult();
                    string[] item = new string[itemMax];
                    int count = 0;
                    // 進捗ダイアログ生成
                    FormProgress form_progress = new FormProgress();
                    form_progress.Owner = this;
                    form_progress.SetPos(count);
                    form_progress.SetMax(listFile.SelectedIndices.Count);
                    form_progress.Show();
                    Application.DoEvents();
                    for(int no = 0 ; no < listFile.SelectedIndices.Count ; no++) {
                        int item_no = listFile.SelectedIndices[no];
                        item[itemSrc] = FileInfoList[item_no].FullName;
                        item[itemDst] = Path.Combine(dialog.SelectPath, FileInfoList[item_no].Name);
                        form_progress.SetMessage(item[itemSrc], item[itemDst]);
                        form_progress.SetPos(++count);
                        Application.DoEvents();
                        try {
                            // 選択しているファイルをコピー
                            FileInfoList[item_no].CopyTo(item[itemDst], flag);

                            item[itemResult] = "成功";
                            item[itemError] = "";
                        } catch (Exception ex) {
                            item[itemResult] = "失敗";
                            item[itemError] = ex.Message;
                        }
                        form_result.AddItem(item);
                        if(form_progress.Stop) {
                            break;
                        }
                    }
                    form_progress.Close();
                    // 結果ダイアログを表示
                    form_result.ShowDialog(this);
               }
            }
        }

        private void menuItemFileCopy_Click(object sender, EventArgs e)
        {
            // ファイルコピー 上書きなし
            FileCopy(nMtp.FileInfo.CopyFailExists);
        }

        private void MenuItemOverwriteCopy_Click(object sender, EventArgs e)
        {
            // ファイルコピー 上書きあり
            FileCopy(0);
        }

        private void MenuItemNewCopy_Click(object sender, EventArgs e)
        {
            // ファイルコピー 同名ファイル有りの場合新しければコピー
            FileCopy(nMtp.FileInfo.CopyNewer);
        }
        /// <summary>
        /// 仮想リストビューアイテム設定
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listFile_RetrieveVirtualItem(object sender, RetrieveVirtualItemEventArgs e)
        {
            string attr_string = (FileInfoList[e.ItemIndex].Attributes != 0) ? FileInfoList[e.ItemIndex].Attributes.ToString() : "";
            string date_string = (FileInfoList[e.ItemIndex].IsValidLastWriteTime) ? FileInfoList[e.ItemIndex].LastWriteTime.ToString() : "";
            string[] item = { FileInfoList[e.ItemIndex].Name, date_string, FileInfoList[e.ItemIndex].Length.ToString(), attr_string };
            e.Item = new ListViewItem(item);
        }
        
        private void MenuItemDelete_Click(object sender, EventArgs e)
        {
            for(int no = 0 ; no < listFile.SelectedIndices.Count ; no++) {
               int item_no = listFile.SelectedIndices[no];
               try {
                   // ファイル削除
                   FileInfoList[item_no].Delete();
                   MessageBox.Show("削除成功:" + FileInfoList[item_no].Name);
               } catch (Exception ex) {
                   MessageBox.Show("削除失敗:" + FileInfoList[item_no].Name + "\n" + ex.Message);
               }
            }
            GetFile(CurrentPath);
        }
    }
}
